
options validvarname=upcase;

proc format;
value age
   0   = '1'
 1-4   = '2'
 5-14  = '3'
15-24  = '4'
25-34  = '5'
35-44  = '6'
45-54  = '7'
55-64  = '8'
65-74  = '9'
75-84  = '10'
85-high = '11'
;
run;


*filename vsd ftp 'c:\vs\deaths\data\dthsta00.txt' host='<ip address>' user='<username>' pass='<password>';

filename vsd 'c:\vs\deaths\data\dthsta00.txt';

data deaths (drop=mcd units);

if mod(_n_,10000) eq 0 then put _n_=;

infile vsd;

input
  @005 county   $2.
  @007 mcd      $2.
  @023 units    $1.
  @035 cause    $4.
  @066 age   ??  3.
;

if (county ge '01' and county le '61') or county eq '70';
if county eq '70' then county = mcd;
if units ge '2' and units le '5' then age = 0;

run;

*
count deaths within each county and age group
;

proc freq data=deaths;
table county*age /  noprint out=dtab1;
*table county*age / noprint out=dtab1 (drop=percent rename=(count=deaths));
*table county*age / noprint out=dtab1 sparse;
where age ne .;
format age age.;
run;

*
what is in the data set 
what are the values of the variable age
some counties do not have deaths in each age group
;

title "DATA SET DTAB1 - FROM PROC FREQ";
proc print data=dtab1;
*where age eq 11;
*format _all_;
run;

title "DATA SET DTAB1 - PROC CONTENTS";
proc contents data=dtab1;
run;

*
another way to creat a data set with counts;
;

proc summary data=deaths;
*proc summary data=deaths nway;
class county age;
output out=dtab2;
format age age.;
run;

*
what is in the data set produced with PROC SUMMARY
extra output (totals for each CLASS variable)
some counties do not have deaths in each age group
;

title "DATA SET DTAB2 - FROM PROC SUMMARY";
proc print data=dtab2;
*format _all_;
run;

*
use options to force ZERO counts into the data set
;

proc summary data=deaths completetypes;
class county;
class age / preloadfmt;
output out=dtab3;
format age age.;
*limit deaths to a specific cause - ICD-10 cancer deaths all start with 'C';
*where cause eq : 'C';
*heart attack - 'acute myocardial infarction';
*where cause eq : 'I21';
run;

*
what is in the data set when COMPLETETYPES and PRELOADFMT are used
;

title "DATA SET DTAB3 - FROM PROC SUMMARY WITH COMPLETE TYPES OPTION";
proc print data=dtab3;
*format _all_;
run;

*
rearrange the data set - one record per county
;

proc transpose data=dtab3 out=dtrans3 (drop=_:) prefix=d;
var _freq_;
id age;
by county;
where age ne .;
*format _all_;
run;

*
what is in the transposed data set
;

title "DATA SET DTRANS3";
proc print data=dtrans3 noobs;
run;

*
create population data
use a SAS data set
;

libname sf1 'c:\census\sf1\datasets';
*libname sf1 'e:\census\sf1\v8';

* 
what is in the census data set
need SF1 documentation
;

title "VARIABLES IN DATA SET PCT12";
proc contents data=sf1.pct12;
run;

*
create populations in 11 age groups
;

data pop1 (keep=county p0-p11);
set sf1.pct12;
where sumlev eq '050';
p0  = pct012001;
p1  = sum (of pct012003 pct012107);
p2  = sum (of pct012004-pct012007 pct012108-pct012111);
p3  = sum (of pct012008-pct012017 pct012112-pct012121);
p4  = sum (of pct012018-pct012027 pct012122-pct012131);
p5  = sum (of pct012028-pct012037 pct012132-pct012141);
p6  = sum (of pct012038-pct012047 pct012142-pct012151);
p7  = sum (of pct012048-pct012057 pct012152-pct012161);
p8  = sum (of pct012058-pct012067 pct012162-pct012171);
p9  = sum (of pct012068-pct012077 pct012172-pct012181);
p10 = sum (of pct012078-pct012087 pct012182-pct012191);
p11 = sum (of pct012088-pct012105 pct012192-pct012209);
*if p0 ne sum (of p1-p11) then put 'mistake ' county=;
run;

*
what is in the population data set
look at the county numbers
;

title "POPULATION DATA SET";
proc print data=pop1 noobs;
var county;
sum p0-p11;
run;

*
add a state total to the data set
;

proc summary data=pop1;
class county;
var p0-p11;
output out=tpop1 (drop=_:) sum=;
run;

*
what is in the population data set
look at the county numbers
;

title "STATE TOTAL ADDED USING A CLASS VARIABLE";
proc print data=tpop1 noobs;
var county p0-p11;
run;

*
another way to create age groups
;

data pop2 (keep=county p0-p11);
array m(0:102) pct012003-pct012105;
array f(0:102) pct012107-pct012209;
array p(11);
array start(11) _temporary_ (0 1  5 15 25 35 45 55 65 75  85);
array stop(11)  _temporary_ (0 4 14 24 34 44 54 64 74 84 102);
set sf1.pct12;
where sumlev eq '050';
do i=1 to 11;
   p(i) = 0;
do j=start(i) to stop(i);
   p(i) + m(j) + f(j);
end;
end;
p0 = sum (of p1-p11);
run;

*
what is in the data set
;

title "POPULATION DATA SET CREATED USING ARRAYS AND LOOPS";
proc print data=pop2 noobs;
var county;
sum p0-p11;
run;

*
change county numbers in the DEATH data set to FIPS numbers
use a format in the format library
add total deaths to each observation
;

proc format library=library;
select $cog2fip;
run;

data dtrans3;
set dtrans3 (rename=(county=temp));
county = put(temp,$cog2fip.);
if county eq '999' then county = '';
d0 = sum(of d1-d11);
drop temp;
run;

*
what is in the DEATH data set
;

title "DATA SET DTRANS3 - FIPS COUNTY NUMBER, TOTAL DEATHS ADDED";
proc print data=dtrans3 noobs;
var county d0-d11;
run;

*
merge DEATHS and POPULATION
sort the DEATHS first, by county number
;

proc sort data=dtrans3;
by county;
run;

data dth_pop;
merge dtrans3 tpop1;
by county;
run;

title "DATA SET CONTAING DEATHS AND POULATIONS";
proc print data=dth_pop noobs;
var county d0 p0 d1-d11 p1-p11;
run;

* 
use an data step to comput an age-adjusted death rate
;

data rates (keep=county d0 p0 r_:);
*** year 2000;
array adj(11) _temporary_ (.013818 .055317 .145565 .138646 .135573 .162613
                           .134834 .087247 .066037 .044842 .015508);

*** year 1940;
*array adj(11) _temporary_ (.015343 .064718 .170355 .181677 .162066 .139237
                           .117811 .080294 .048426 .017303 .002770);

array d(11);
array p(11);

set dth_pop;

r_crude = 100000 * d0 / p0;
r_adjusted = 0;

do j=1 to 11;
   r_adjusted + adj(j)*d(j)/p(j);
end;

r_adjusted = 100000*r_adjusted;

label
d0 = 'DEATHS'
p0 = 'POPULATION'
;

run;

*
use a recursive format to label the blank county
add a rule to an existing format
;

title "FORMAT ALREADY IN FORMAT LIBRARY";
proc format library=library;
select $fip2nam;
run;

proc format;
value $county
'   ' = 'NEW YORK STATE'
other = [$fip2nam40.]
;
run;

title "RECURSIVE FORMAT";
proc format;
select $county;
run;

title "CRUDE AND AGE-ADJUSTED DEATH RATES PER 1,000 POPULATION, NYS 2000";
proc print data=rates noobs label;
var county d0 p0 r_crude r_adjusted;
format county $county. d0 p0 comma10. r_: 6.1;
run;

*
add RANKS
use DESCENDING - #1 is worst
;

proc rank data=rates out=ranks descending ties=low;
where county ne '';
var r_crude r_adjusted;
ranks rank_c rank_a;
*label 
rank_c = "CRUDE RANK"
rank_a = "ADJUSTED RANK"
;
run;

title "CRUDE AND AGE-ADJUSTED DEATH RATES (+ RANK) PER 1,000 POPULATION, NYS 2000";
proc print data=ranks noobs label;
var county d0 p0 r_crude r_adjusted rank_c rank_a;
format county $county. d0 p0 comma10. r_: 6.1;
run;

*
use ODS to create either HTML or PDF output
;

ods listing close;
ods html file='z:\age_adjust.htm' style=barrettsblue;

title "CRUDE AND AGE-ADJUSTED DEATH RATES (+ RANK) PER 1,000 POPULATION, NYS 2000";
proc print data=ranks noobs label;
var county d0 p0 r_crude r_adjusted rank_c rank_a;
format county $county. d0 p0 comma10. r_: 6.1;
run;

ods html close;
ods listing;

ods listing close;
ods pdf file='z:\age_adjust.pdf' notoc ;

title "CRUDE AND AGE-ADJUSTED DEATH RATES (+ RANK) PER 1,000 POPULATION, NYS 2000";
proc print data=ranks noobs label;
var county d0 p0 r_crude r_adjusted rank_c rank_a;
format county $county. d0 p0 comma10. r_: 6.1;
run;

ods pdf close;
ods listing;

*
look at a plot of the RANKS
points close to DIAGONAL not affected much by adjustment
;

goptions reset=all ftext='Arial/bo' gunit=pct htext=2.5;

symbol f='Arial' v='*' h=5;

%annomac;
data line;
retain xsys ysys '2';
%LINE (0, 0, 70, 70, red, 2, 2);
run;

title "COMPARISON OF CRUDE AND ADJUSTED RANKS";
proc gplot data=ranks;
plot rank_a*rank_c / annotate=line;
*plot r_adjusted*r_crude;
label 
rank_c = 'CRUDE'
rank_a = 'ADJ'
;
run;
quit;


************************************************************************************
ANOTHER APPROACH TO CREATING OUTPUT DATA SETS - USE THE OUTPUT DELIVERY SYSTEM (ODS)
************************************************************************************;

title;

*
TRACE writes the names of sections of PROC output to the LOG
;

ods trace on;

proc freq data=deaths;
table county*age / norow nocol nopercent;
where age ne .;
format age age.;
run;

ods trace off;

*** DIGRESSION ***;

*
TRACE writes the names of sections of PROC output on the OUTPUT
;

ods trace on /listing;

proc univariate data=deaths;
var age;
where cause eq : 'C';
run;

ods trace off;

*
use ODS select to limit PROC output
;

proc univariate data=deaths;
var age;
where cause eq : 'C';
ods select quantiles;
run;

*
direct PROC output to a data set
;

ods listing close;
ods output crosstabfreqs=dtab4;

proc freq data=deaths;
table county*age;
where age ne .;
format age age.;
run;

ods listing;

*
what is in the data set
notice the ZERO frequencies
notice the extra variables
notice the summary observations (similar to PROC SUMMARY)
;

proc print data=dtab4 noobs;
*format _all_;
run;

*
create a DEATHS data set with one observation per county
;

ods listing close;
ods output crosstabfreqs=dtab5 (keep=county age frequency);

proc freq data=deaths;
table county*age;
where age ne .;
format age age.;
run;

proc sort data=dtab5;
by county;
run;

proc transpose data=dtab5 out=dtrans5 (drop=_:) prefix=d;
var frequency;
id age;
by county;
where age ne .;
*format _all_;
run;

ods listing;

proc print data=dtrans5 noobs;
run;
